@@ -20,18 +20,28 @@ add the following lines to /etc/modules
  - i2c-dev
  - i2c-bcm2708
 
-create a virtual environment (venv) and use the pip install -r requirements.txt
+Create a virtual environment (venv) and use the pip install -r requirements.txt
 to install the dependencies
 
 """
 import time
 import logging
 import Adafruit_PCA9685
-# from constants import Channel
 from .channel import Channel
 logging.basicConfig(level=logging.CRITICAL)
 logging.propagate = False
 
+chans = { 'LEFT_LEG_FRONT': 0,    # channel 0
+        'LEFT_LEG_BACK': 2,     # channel 2
+        'RIGHT_LEG_FRONT': 6,   # channel 6
+        'RIGHT_LEG_BACK': 4,    # channel 4
+
+        'LEFT_FOOT_FRONT':  1,   # channel 1
+        'LEFT_FOOT_BACK': 3,    # channel 3
+        'RIGHT_FOOT_FRONT': 7,  # channel 7
+        'RIGHT_FOOT_BACK': 5,   # channel 5
+}
+
 # Set DEBUG to True using the debug property
 DEBUG = False
 
@@ -92,8 +102,8 @@ def set_servo_pulse(channel, pulse):
         except (RuntimeError) as ex:
             logging.warning(
                 """Failed to set pwm
-                 - did the driver initialize correctly? %s""", ex)
-            # print("Failed to set pwm - did the driver initialize correctly?")
+                    - did the driver initialize correctly? %s""", ex)
+        # print("Failed to set pwm - did the driver initialize correctly?")
 
         return True
 
@@ -104,7 +114,7 @@ def set_servo_pulse(channel, pulse):
     return False
 
 
-class Leg(object):
+class Leg():
     """
     provides a model of a limb (for either a foot or a leg)
     """
@@ -116,8 +126,8 @@ class Leg(object):
     __currentangle = 0
     __invert = False
     __leg_angle = 0
-    __leg_minAngle = 0
-    __leg_maxAngle = 180
+    __leg_minangle = 0
+    __leg_maxangle = 180
 
     @property
     def angle(self):
@@ -159,6 +169,69 @@ class Leg(object):
             self.__swingangle = (self.__leg_maxangle - self.__leg_minangle) / 2
         self.__currentangle = self.__bodyangle
 
+    @property
+    def leg_minangle(self):
+        """ Gets the minimum limb angle """
+        return self.__leg_minangle
+
+    @leg_minangle.setter
+    def leg_minangle(self,value:int) -> bool:
+        """ Sets the minimum limb angle """
+
+        if not isinstance(value, int):
+            print("Oops Limb Minimum Angle setter was expected the value to be an integer, \
+                please try again but with a valid number between 0 and 180.")
+            return False
+
+        if 0<= value <= 180:
+            self.__leg_minangle = value
+            return True
+
+        return False
+
+    @property
+    def leg_maxangle(self):
+        """ Gets the maximum limb angle """
+        return self.__leg_maxangle
+
+    @leg_maxangle.setter
+    def leg_maxangle(self,value:int) -> bool:
+        """ Sets the maximum limb angle """
+
+        if not isinstance(value, int):
+            print("Oops Limb Maximum Angle setter was expected the value to be an integer, \
+                please try again but with a valid number between 0 and 180.")
+            return False
+
+        if 0<= value <= 180:
+            self.__leg_maxangle = value
+            return True
+
+        return False
+
+    @property
+    def channel(self):
+        """ Returns the PCA9685 channel this servo/limb uses"""
+        return self.__channel
+
+    @channel.setter
+    def channel(self, value:int) -> bool:
+        """ Set the channel for this servo/limb """
+
+        if not isinstance(value, int):
+            print("Oops Limb channel setter was expected the value to be an integer, \
+                please try again but with a valid number between 0 and 15.")
+            return False
+
+        # Check its a valid channel number
+        if 0 <= value <= 15:
+            self.__channel = value
+            return True
+        print("Oops Limb channel setter was expected the value to be an integer, \
+             between 0 and 15.")
+        return False
+
+
     @property
     def invert(self):
         """ returns the invert value """
@@ -170,12 +243,6 @@ class Leg(object):
         self.__invert = value
 
 
-    def setdefault(self):
-        """
-        DEPRICATED METHOD - USE .default
-        """
-        print("This method is depricated, use .default instead")
-
     def default(self):
         """
         Sets the limb to the default angle, by subtracting the maximum and
@@ -184,11 +251,6 @@ class Leg(object):
         self.angle = self.__leg_maxangle - self.__leg_minangle
         self.__currentangle = self.__leg_maxangle - self.__leg_minangle
 
-    def setbody(self):
-        """
-        DEPRICATED METHOD - USE .body
-        """
-        print("Function setbody: this method is depricated, use .body instead")
 
     def body(self):
         """
@@ -202,9 +264,6 @@ class Leg(object):
             self.__bodyangle = self.__leg_maxangle
         self.__currentangle = self.__bodyangle
 
-    def setstrech(self):
-        """ DEPRICATED METHOD - USE .stretch """
-        print("this method is depricated, use .stretch instead")
 
     def stretch(self):
         """
@@ -218,11 +277,16 @@ class Leg(object):
             self.__stretchangle = self.__leg_minangle
         self.__currentangle = self.__stretchangle
 
+    def identify(self):
+        """ Wiggles the limb between the angles of 85 - 95 for a couple of seconds """
 
-    def setswing(self):
-        """ DEPRICATED - Use Swing instead"""
-        print("This function is depricated use swing instead")
-
+        # wiggle
+        for _ in range(1,5):
+            self.angle = 80
+            time.sleep(.25)
+            self.angle = 100
+            time.sleep(.25)
+        self.angle = 90
 
     def swing(self):
         """
@@ -269,24 +333,18 @@ class Leg(object):
         print(self.__channel)
         print(self.name)
 
-    def setangle(self, value):
-        """
-        DEPRICATED METHOD - USE .angle
-        """
-        print(f'This method is depricated - use .angle instead to set angle to {value}')
-
     @angle.setter
-    def angle(self, user_angle):
+    def angle(self, user_angle:int) -> bool:
         """
         Works out the value of the angle by mapping the leg_min and leg_max to
         between 0 and 180 degrees, then moves the limb to that position
         """
         pulse = 0
 
-        if user_angle >= 0 and user_angle <= 180:
+        if 0 <= user_angle <= 180:
             self.__leg_angle = user_angle
             # Check the angle is within the boundaries for this limb
-            if user_angle >= self.__leg_minangle and user_angle <= self.__leg_maxangle:
+            if self.__leg_minangle <= user_angle <= self.__leg_maxangle:
                 mapmax = self.__leg_max - self.__leg_min
                 percentage = (float(user_angle) / 180) * 100
                 pulse = int(((float(mapmax) / 100) *
@@ -308,10 +366,7 @@ class Leg(object):
             # the range (leg_minAngle and leg_maxAngle)
             logging.warning("Warning: angle was outside of bounds for this leg")
             return False
-        else:
-            logging.warning("Warning: angle was less than 0 or greater \
-            than 180.")
-            return False
+        return False
 
     def untick(self):
         """ Used to walk backwards """
@@ -322,7 +377,7 @@ class Leg(object):
                 self.angle = self.__currentangle
                 return False
             return True
-        elif self.__name == "left_leg_back" or self.__name == "left_leg_front":
+        if self.__name == "left_leg_back" or self.__name == "left_leg_front":
             if self.__currentangle >= self.__leg_minangle:
                 self.__currentangle -= 2
                 # print self.name, "setting angle to ", self.currentAngle
@@ -343,7 +398,7 @@ class Leg(object):
                 self.angle = self.__currentangle
                 return False
             return True
-        elif self.__name == "right_leg_front" or self.__name == "right_leg_back":
+        if self.__name == "right_leg_front" or self.__name == "right_leg_back":
             if self.__currentangle >= self.__leg_minangle:
                 self.__currentangle -= 2
                 # print self.name, "setting angle to ", self.currentAngle
@@ -353,16 +408,16 @@ class Leg(object):
         return True
 
     @property
-    def name(self):
+    def name(self)->str:
         """ gets the Robot name """
         return self.__name
 
     @name.setter
-    def name(self, name):
+    def name(self, name:str):
         """ Sets the Robot name """
         self.__name = name
 
-class SmarsRobot(object):
+class SmarsRobot():
     """
     This is used to model the robot, its legs and its sensors
     """
@@ -384,7 +439,9 @@ class SmarsRobot(object):
     # setup two arrays, one for legs, and one for feet
     __legs = []
     __feet = []
-    __name = ""  # the friendly name for the robot - used in console messages.
+
+    # the friendly name for the robot - used in console messages
+    __name = ""
 
     # debug status, default if off / False
     __debug = False
@@ -410,14 +467,78 @@ class SmarsRobot(object):
                     leg_minangle=9, leg_maxangle=90, invert=True))
     # print "number of legs", len(legs)
 
+    def identify(self, channel:int)->str:
+        """ Identies the limb by the channel passed in, returns the limb name """
+        for limb in self.__feet:
+            if limb.channel == channel:
+                print("limb found in Feet")
+                limb.identify()
+                return limb.name
+        for limb in self.__legs:
+            if limb.channel == channel:
+                print("limb found in Legs")
+                limb.identify()
+                return limb.name
+        return "Limb not found"
+
+    def set_limb_channel(self, limb_name, channel)->bool:
+        """ Sets the limb name to the channel provided, returns True if complete, False if not """
+
+        index = chans.get(limb_name)
+        print(index)
+
+        found = False
+        for limb in self.__legs:
+            if limb.name == limb_name:
+                self.__legs[index].channel = channel
+                found = True
+        for limb in self.__feet:
+            if limb.name == limb_name:
+                self.__feet[index].channel = channel
+                found = True
+
+        if found:
+            # need to check which other limb still has this number
+            for limb in self.__legs:
+                if (limb.channel == channel) and (limb.name != limb_name):
+                    print("Remember to change", limb.name, "as this is still \
+                        using channel", channel)
+            return True
+        print("Limb name not found, sorry")
+        return False
+
+
     @property
-    def debug(self):
+    def config(self)->dict:
+        """ Get the current limb configuration as a dictionary of settings"""
+        limb_config = []
+        temp_limb = []
+        for limb in self.__feet:
+            temp_limb = {'name': limb.name,
+                         'channel': limb.channel,
+                         'invert':limb.invert,
+                         'min_angle':limb.leg_minangle,
+                         'max_angle':limb.leg_maxangle
+                         }
+            limb_config.append(temp_limb)
+        for limb in self.__legs:
+            temp_limb = {'name': limb.name,
+                         'channel': limb.channel,
+                         'invert':limb.invert,
+                         'min_angle':limb.leg_minangle,
+                         'max_angle':limb.leg_maxangle
+                         }
+            limb_config.append(temp_limb)
+        return limb_config
+
+    @property
+    def debug(self)->bool:
         """ returns the debug status """
         return self.__debug
 
 
     @debug.setter
-    def debug(self, value):
+    def debug(self, value:bool):
         if value:
             self.__debug = True
         elif not value:
@@ -442,14 +563,14 @@ class SmarsRobot(object):
 
 
     @property
-    def name(self):
+    def name(self)->str:
         """
         gets the robots name
         """
         return self.__name
 
     @name.setter
-    def name(self, name):
+    def name(self, name:str):
         """
         Sets the robots name, used for displaying console messages.
         """
@@ -458,15 +579,6 @@ class SmarsRobot(object):
         if self.debug:
             logging.info("changed name to %s", name)
 
-    def setname(self,name):
-        """
-        Depricated - use the .name property
-        """
-        print("Depricated - use the .name property")
-        if self.debug:
-            print("actually setting the name property")
-            self.name = name
-
     def leg_reset(self):
         """
         used to reset all the legs
@@ -555,13 +667,13 @@ class SmarsRobot(object):
         # move legs one at a time back to swing position
         self.swing()
 
-    def forward(self, steps=None):
+    def forward(self, steps:int=None):
         """ Move the Robot Forward """
         if steps is None:
             steps = 1
         self.walkforward(steps)
 
-    def backward(self, steps=None):
+    def backward(self, steps:int=None):
         """ Move the Robot Backward """
         if steps is None:
             steps = 1
@@ -583,7 +695,7 @@ class SmarsRobot(object):
         print("body()")
         print("default()")
 
-    def walkforward(self, steps=None):
+    def walkforward(self, steps:int=None):
         """
         Used to move the robot forward
         """
@@ -678,10 +790,15 @@ class SmarsRobot(object):
                     self.__feet[tick_count].up()
                     time.sleep(SLEEP_COUNT)
 
-    def clap(self, clap_count):
+    def clap(self, clap_count=None):
         """  Clap front two hands (the sound of two hands clapping) """
         chan = Channel()
 
+        print("Clapping")
+
+        if clap_count is None:
+            clap_count = 1
+
         self.sit()
         # self.feet[left_foot_front].up()
         # self.feet[right_foot_front].up()
@@ -694,9 +811,14 @@ class SmarsRobot(object):
             time.sleep(SLEEP_COUNT * 2)
         self.stand()
 
-    def wiggle(self, wiggle_count):
+    def wiggle(self, wiggle_count=None):
         """ Wiggle butt """
 
+        print("Wiggling")
+
+        if wiggle_count is None:
+            wiggle_count = 1
+
         chan = Channel()
 
         self.sit()
@@ -727,7 +849,7 @@ class SmarsRobot(object):
         telemetry.append(["right_foot_back", self.__legs[chan.RIGHT_FOOT_BACK].angle])
         return telemetry
 
-class CommandHistory(object):
+class CommandHistory():
     """ models the command history object """
 
     # Private property History
@@ -749,15 +871,7 @@ class CommandHistory(object):
         """ gets all command history """
         return self.__history
 
-    def get_history(self):
-        """ DEPRICATED - use .history property"""
-        print("This function is depricated - use .history property")
-
     @property
     def last_ten(self):
         """ get last 10 command history """
         return self.__history[-10:]
-
-    def get_last_ten(self):
-        """ DEPRICATED - Use .last_ten property """
-        print("This function is deprecated - use .last_ten property instead")
